/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Serial.c
**  Abstract :	This file implements device driver for Serial module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/10/29
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Serial.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
volatile UCHAR	gIic0MasterStatusFlag;
volatile UCHAR	gIic0SlaveStatusFlag;
volatile USHORT	gIic0TxCnt;
volatile UCHAR*	gpIic0TxAddress;
volatile UCHAR*	gpIic0RxAddress;
volatile USHORT	gIic0RxLen;
volatile USHORT	gIic0RxCnt;
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function initializes IIC0.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void IIC0_Init( void )
{
	IICE0 = 0;		/* IIC0 disable */
	IICMK0 = 1;	/* INTIIC0 disable */
	IICIF0 = 0;	/* clear INTIIC0 interrupt flag */
	/* Set INTIIC0 high priority */
	IICPR0 = 0;
	/* Set clock pin for IIC0 */
	PM6 &= ~0x1;
	P6 &= ~0x1;
	/* Set data transfer I/O pin for IIC0 */
	PM6 &= ~0x2;
	P6 &= ~0x2;
	IICCL0 = IICCL0_INITIALVALUE | IIC0_CLOCK0;
	IICX0 = IIC0_EXPENSION0;
	SVA0 = IIC0_MASTERADDRESS;
	STCEN = 1;
	IICRSV = 1;
	SPIE0 = 0;
	WTIM0 = 1;
	ACKE0 = 1;
	IICMK0 = 0;
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function stops IIC0.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void IIC0_Stop( void )
{
	SPT0 = 1;	/* set stopcondition flag */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function starts IIC0 send data as master mode.
**
**	Parameters:
**		adr :	set address for select slave
**		txbuf :	transfer buffer pointer
**		txnum :	buffer size
**		wait :	wait for start condition
**
**	Returns:
**		MD_OK
**		MD_ERROR
**
**-----------------------------------------------------------------------------
*/
MD_STATUS IIC0_MasterSendStart( UCHAR adr, UCHAR* txbuf, USHORT txnum, UCHAR wait )
{
	MD_STATUS status = MD_OK;
	
	IICE0 = 1;
	LREL0 = 1;
	/* Bus check */
	IICMK0 = 1;
	if( IICBSY == 1 )
	{
		IICMK0 = 0;	/* bus busy */
		status = MD_ERROR;
	}
	else
	{
		/* Trigger check */
		if( SPT0 == 1 || STT0 == 1)
		{
			IICMK0 = 0;
			status = MD_ERROR;
		}
		else
		{
			STT0 = 1;	/* IIC0 send start condition */
			IICMK0 = 0;
			/* Wait */
			while( wait-- );
			/* Set parameter */
			gIic0TxCnt = txnum;
			gpIic0TxAddress = txbuf;
			gIic0MasterStatusFlag = IIC0_MASTER_FLAG_CLEAR;
			adr &= ~0x01;	/* send mode */
			IIC0 = adr;		/* send address */
		}
	}
	
	return (status);
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function starts IIC0 receive data as master mode.
**
**	Parameters:
**		adr :	set address for select slave
**		rxbuf :	receive buffer pointer
**		rxnum :	buffer size
**		wait :	wait for start condition
**
**	Returns:
**		MD_OK
**		MD_ERROR
**
**-----------------------------------------------------------------------------
*/
MD_STATUS IIC0_MasterReceiveStart( UCHAR adr, UCHAR* rxbuf, USHORT rxnum, UCHAR wait )
{
	MD_STATUS status = MD_OK;
	
	IICE0 = 1;
	LREL0 = 1;
	/* Bus check */
	IICMK0 = 1;
	if( IICBSY == 1 )
	{
		IICMK0 = 0;			/* bus busy */
		status = MD_ERROR;
	}
	else
	{
		/* Trigger check */
		if( SPT0 == 1 || STT0 == 1)
		{
			IICMK0 = 0;
			status = MD_ERROR;
		}
		else
		{
			STT0 = 1;		/* IIC0 start condition */
			IICMK0 = 0;
			/* Wait */
			while( wait-- );
			/* Set parameter */
			gIic0RxLen = rxnum;
			gIic0RxCnt = 0;
			gpIic0RxAddress = rxbuf;
			gIic0MasterStatusFlag = IIC0_MASTER_FLAG_CLEAR;
			adr |= 0x01;	/* receive mode */
			IIC0 = adr;		/* receive address */
		}
	}
	
	return (status);
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


