/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Timer.c
**  Abstract :	This file implements device driver for Timer module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/10/29
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Timer.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function initializes TM50.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM50_Init( void )
{
	TCE50 = 0;			/* TM50 counter disable */
	TMMK50	= 1;		/* INTTM50 interrupt disabled */
	TMIF50	= 0;		/* clear interrupt flag */
	TMPR50	= 0;		/* set high priority level */
	TCL50 = TM5_INTER_CLOCK_5;
	 /* TM50 interval mode */
	CR50 = TM50_CR50_VALUE;
	TMC50 = TM5_TMC5_INITIALVALUE | TM5_OPT_MODE_OTHER | TM5_OUTPUT_FF_ENABLE;	
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function starts TM50 counter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM50_Start( void )
{
	TMMK50	= 0;		/* INTTM50 interrupt enabled */
	TCE50 = 1;			/* count operation start */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function stops TM50 counter.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM50_Stop( void )
{
	TCE50 = 0;			/* TM50 counter disable */
	TMMK50	= 1;		/* INTTM50 interrupt disabled */
	TMIF50	= 0;		/* clear interrupt flag */
}

/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function changes TM50 compare value.
**
**	Parameters:
**		value :	value for compare register
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
void TM50_ChangeTimerCondition( UCHAR value )
{
	CR50 = value;
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */


