/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2011
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	 main.c
**  Abstract :	This file implements main function.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0512_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2011/10/28
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Int.h"
#include "Ad.h"
#include "Serial.h"
#include "Timer.h"
#include "Port.h"
#include "System.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function  implements main function.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
#define	LCD_CMD_WAIT		400
#define	LCD_BIT_WAIT		10
#define	PRESET_MAX			20
#define	VOL_COUNT_SET		10
#define	FREQ_MIN			760
#define	FREQ_MAX			1080
#define FREQ_MANUAL_DIV		7

#define	KYOKU_INIT			500
#define	KYOKU_ON			490

#define	LCD_DB4				P2.0
#define	LCD_DB5				P2.1
#define	LCD_DB6				P2.2
#define	LCD_DB7				P2.3
#define	LCD_RS				P13.0
#define	LCD_E				P14.0
#define	KEY_POWER			P1.3
#define	KEY_UP				P1.0
#define	KEY_DOWN			P1.1
#define	KEY_MODE			P1.2
#define	POWER_CONT			P12.2
#define	FM_SEN				P3.1
#define	FM_SCLK				P3.2
#define	FM_SDIO				P3.3
#define	FM_RESET			P4.0

unsigned short	ad_lev_in;
unsigned short	vol_level;
unsigned short	vol_level_m;

unsigned int	mhz_no;
unsigned int	mhz_no_m;

unsigned char	mode;

unsigned int	mhz_no_setmode;

unsigned char	eeprom_addr=0xa0;
unsigned char	eeprom_finish;
unsigned char	eeprom_write_data[3];
unsigned char	eeprom_read_data[1];

unsigned char	preset_no;
unsigned int	preset[PRESET_MAX];
unsigned char	preset_no_setmode;
unsigned char	preset_ch[FREQ_MAX-FREQ_MIN+5];

unsigned char	set_count;
unsigned char	tun_flg;

unsigned int	kyoku_count;

unsigned char	count;
unsigned char	rec[12];
unsigned char	count10_flg;
unsigned char	receive_err;
unsigned int	receive_no;

void	wait_time(unsigned int time)
{
	while(time!=0){
		time--;
	}
}
void	preset_ch_init(void)
{
unsigned int	i;
	for(i=0;i<FREQ_MAX-FREQ_MIN+1;i++){
		preset_ch[i]=0xff;
	}
}
/*----------  LCD MODULE ----------*/
void	lcd_db(unsigned char data)
{
	LCD_DB4=data&0x01;
	LCD_DB5=(data&0x02)>>1;
	LCD_DB6=(data&0x04)>>2;
	LCD_DB7=(data&0x08)>>3;
}
void	lcd_wait(void)
{
	wait_time(5000);
}
void	lcd_c_set(void)
{
	LCD_RS=0;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_d_set(void)
{
	LCD_RS=1;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_ini_disp(void)
{
	lcd_xy(4,0);
	lcd_print("     MHz");
}
void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_4keta10(unsigned int count)
{
unsigned char n0,n1,n2,n3;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_mode(unsigned char m)
{
	lcd_xy(0,1);
	switch(m){
		case 1:	lcd_print("TUNE");
				if(tun_flg==1){lcd_print("m");}else{lcd_print("a");}
				break;
		case 0:	lcd_print("CH-");lcd_2keta10(preset_no);break;
		case 2:	lcd_print("SET");lcd_2keta10(preset_no_setmode);break;
	}
}
void	lcd_mhz(unsigned int hz)
{
unsigned char n0,n1,n2,n3;
unsigned int count;

	/*lcd_xy(4,0);*/
	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	if(n3==0){lcd_print(" ");}
	else{lcd_su(n3);}
	lcd_su(n2);
	lcd_su(n1);
	lcd_print(".");
	lcd_su(n0);
}
void	lcd_ch_disp(void)
{
	if(preset_ch[mhz_no-FREQ_MIN]!=0xff){
		lcd_xy(8,1);
		lcd_print("(CH-  )");
		lcd_xy(12,1);
		lcd_2keta10(preset_ch[mhz_no-FREQ_MIN]);
	}
	else{
		lcd_xy(8,1);lcd_print("       ");
	}
}
void	lcd_ch_disp_sub(void)
{
	if(preset_ch[mhz_no-FREQ_MIN]!=0xff){
		lcd_xy(8,1);
		lcd_print("(CH-  )");
		lcd_xy(12,1);
		lcd_2keta10(preset_ch[mhz_no-FREQ_MIN]);
	}
}

void	lcd_kyokumei_clr(void)
{
	lcd_xy(5,1);
	lcd_print("           ");
}

/*----------  MHZ    ----------*/
void	mhz_inc(void)
{
	mhz_no++;
	if(mhz_no==1081){mhz_no=760;}
}

void	mhz_dec(void)
{
	mhz_no--;
	if(mhz_no==759){mhz_no=1080;}
}
void	mhz_inc_10(void)
{
	mhz_no=mhz_no/10;
	mhz_no=mhz_no*10+10;
	if(mhz_no>=1081){mhz_no=760;}
}

void	mhz_dec_10(void)
{
	mhz_no=mhz_no/10;
	mhz_no=mhz_no*10-10;
	if(mhz_no<=759){mhz_no=1080;}
}
void	lcd_kyokumei(unsigned int no)
{
	lcd_xy(6,1);
	switch(no){
		case 771:lcd_print("Kiss-FM   ");break;
		case 796:lcd_print("NHK-FM    ");break;
		case 809:lcd_print("FM-HANAKO ");break;
		case 825:lcd_print("FM OSAKA  ");break;
		case 833:lcd_print("FM COCOLO ");break;
		case 845:lcd_print("FM-802    ");break;
		case 859:lcd_print("Alpha-STA.");break;

		case 779:lcd_print("HARBOR ST.");break;
		case 802:lcd_print("FM-802    ");break;
		case 849:lcd_print("NHK-FM    ");break;
		case 864:lcd_print("FM FUKUI  ");break;
		case 872:lcd_print("Alpha-STA.");break;

		default:lcd_print("          ");break;
	}
}

/*----------  A/D ----------*/
void	ad_input7(void)
{
    WTMK = 1;			/* INTWT Interrupt servicing disabled */
	AD_SelectADChannel(AD_INPUT_CHANNEL_7);
	AD_Read(&ad_lev_in);
    WTMK = 0;			/* INTWT Interrupt servicing enabled */
	
}
unsigned int	vol_yomi(void)
{
unsigned short	sa;

	ad_input7();
	vol_level=ad_lev_in;
	if(vol_level>vol_level_m){
		sa=vol_level-vol_level_m;
	}
	if(vol_level<=vol_level_m){
		sa=vol_level_m-vol_level;
	}
	if(sa>=4){
		vol_level_m=vol_level;
	}
	else{
		vol_level=vol_level_m;
	}
	vol_level=vol_level/FREQ_MANUAL_DIV;
	vol_level=vol_level+FREQ_MIN;
		
	
	return vol_level;
}

/*---------- PRESET ---------*/
unsigned char	preset_next(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	no++;
	if(no>=PRESET_MAX){no=0;}
	for(i=no;i<PRESET_MAX;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=0;i<no;i++){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}

unsigned char	preset_back(unsigned char no)
{
unsigned char i,ret;

	ret=no;
	if(no==0){no=PRESET_MAX-1;}else{no--;}
	for(i=no;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	for(i=PRESET_MAX-1;i>=0;i--){
		if(preset[i]!=0){
			ret=i;
			return ret;
		}
	}
	return ret;
}
void	preset_ch_set(void)
{
unsigned char	i;
unsigned int	mhz;

	for(i=0;i<PRESET_MAX;i++){
		if(preset[i]!=0x00){
			mhz=preset[i];
			preset_ch[mhz-FREQ_MIN]=i;
		}
	}
}

/*---------- FM CHIP ---------*/
void	data_out(unsigned char ad,unsigned int da)
{
unsigned char	i;
unsigned char	nn;
unsigned char	x;

	FM_SEN=0;
	FM_SCLK=0;
	FM_SDIO=0;

	nn=0x60;
	for(i=0;i<5;i++){
		if((nn&0x80)==0x80){
			FM_SDIO=1;
		}
		else{
			FM_SDIO=0;
		}
		nn=nn<<1;
		FM_SCLK=1;
		wait_time(30);
		FM_SCLK=0;
	}

	ad=ad<<4;
	for(i=0;i<4;i++){
		if((ad&0x80)==0x80){
			FM_SDIO=1;
		}
		else{
			FM_SDIO=0;
		}
		ad=ad<<1;
		FM_SCLK=1;
		wait_time(30);
		FM_SCLK=0;
	}

	for(i=0;i<16;i++){
		if((da&0x8000)==0x8000){
			FM_SDIO=1;
		}
		else{
			FM_SDIO=0;
		}
		da=da<<1;
		FM_SCLK=1;
		wait_time(30);
		FM_SCLK=0;
	}

	FM_SDIO=0;
	FM_SCLK=0;
	FM_SEN=1;
	
	FM_SCLK=1;
	wait_time(30);
	FM_SCLK=0;
}

void	fm_chip_init(void)
{
	FM_RESET=0;
	FM_SEN=0;
	FM_SDIO=0;
	wait_time(300);
	FM_RESET=1;
	wait_time(300);
	FM_SEN=1;
	wait_time(300);
	
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);

	data_out(0x02,0x4001);
	/*data_out(0x03,0x8000);
	wait_time(5000);
	data_out(0x03,0x8000);*/
	data_out(0x04,0x0800);
	data_out(0x05,0x005f);
}
void	fm_data_set(unsigned int data)
{
unsigned int	dd;

	dd=data-760;

	data_out(0x03,dd|0x8000);
	wait_time(10000);
	data_out(0x03,dd);
	/*data_out(0x04,0x0800);
	data_out(0x05,0x005f);*/
}
void	mute(unsigned char f)
{
	switch(f){
		case 1:data_out(0x02,0x0001);break;
		case 0:data_out(0x02,0x4001);break;
	}
}

/*---------- EEPROM ---------*/
void	eeprom_write(unsigned char ad1,unsigned char ad2,unsigned char d)
{
	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	eeprom_write_data[2]=d;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 3,1);
	while (!eeprom_finish){};
}
unsigned char	eeprom_read(unsigned char ad1,unsigned char ad2)
{
unsigned char	data;

	eeprom_write_data[0]=ad1;
	eeprom_write_data[1]=ad2;
	eeprom_finish = 0 ;
	IIC0_MasterSendStart(eeprom_addr, eeprom_write_data, 2,1);
	while (!eeprom_finish){};

	eeprom_finish = 0 ;
	IIC0_MasterReceiveStart(eeprom_addr, eeprom_read_data,1,10);
	while (!eeprom_finish) {};
	
	data=eeprom_read_data[0];
	
	return data;
}
void	wait_eeprom(void)
{
		wait_time(1000);
}
void	eeprom_write_sub(void)
{
unsigned char	i;

	for(i=0;i<PRESET_MAX;i++){
		eeprom_write(0,0x10+i*2,(unsigned char)(preset[i]%256));wait_eeprom();
		eeprom_write(0,0x11+i*2,(unsigned char)(preset[i]/256));wait_eeprom();
	}
}

void	eeprom_init(void)
{
unsigned char	i;

		preset[0]=FREQ_MAX;

		for(i=1;i<PRESET_MAX;i++){
			preset[i]=0;
		}

		preset[1]=771;preset[2]=796;
		preset[3]=809;preset[4]=825;
		preset[5]=833;preset[6]=845;
		preset[7]=859;preset[8]=779;
		preset[9]=802;preset[10]=849;
		preset[11]=864;preset[12]=872;

		mhz_no=FREQ_MIN;

		eeprom_write(0,0,0xa5);wait_eeprom();
		
		eeprom_write_sub();

		eeprom_write(0,0x0a,(unsigned char)(mhz_no%256));wait_eeprom();
		eeprom_write(0,0x0b,(unsigned char)(mhz_no/256));wait_eeprom();
		eeprom_write(0,0x08,preset_no);wait_eeprom();
		eeprom_write(0,0x05,mode);wait_eeprom();
}

void	eeprom_check(void)
{
unsigned int sub0,sub1;
unsigned char	i;

	if(eeprom_read(0,0)==0xa5){
		for(i=0;i<PRESET_MAX;i++){
			sub0=(unsigned int)eeprom_read(0,0x10+i*2);wait_eeprom();
			sub1=(unsigned int)eeprom_read(0,0x11+i*2);wait_eeprom();
			preset[i]=sub0+sub1*256;
		}
		

		sub0=(unsigned int)eeprom_read(0,0x0a);wait_eeprom();
		sub1=(unsigned int)eeprom_read(0,0x0b);wait_eeprom();
		mhz_no=sub0+sub1*256;
		preset_no=eeprom_read(0,0x08);wait_eeprom();
		mode=eeprom_read(0,0x05);wait_eeprom();
	}
	else{
		eeprom_init();
	}
}
void	write_direct(void){

		eeprom_write(0,0x0a,(unsigned char)(mhz_no%256));wait_eeprom();
		eeprom_write(0,0x0b,(unsigned char)(mhz_no/256));wait_eeprom();
		eeprom_write(0,0x08,preset_no);wait_eeprom();
		eeprom_write(0,0x05,mode);wait_eeprom();
}
void	write_direct_set(unsigned char	no){

		eeprom_write(0,0x10+no*2,(unsigned char)(preset[no]%256));wait_eeprom();
		eeprom_write(0,0x11+no*2,(unsigned char)(preset[no]/256));wait_eeprom();
}
/*---------------------------*/
/*---------- receive signal check ----------*/
void	receive_hantei10(void){

	receive_err=0;
	if((rec[1]>=57)&&(rec[1]<=69)){}else{receive_err=1;return;}
	
	receive_no=0;
	if((rec[2]>=26)&&(rec[2]<=39)){receive_no=receive_no | 0x01;}
	if((rec[3]>=26)&&(rec[3]<=39)){receive_no=receive_no | 0x02;}
	if((rec[4]>=26)&&(rec[4]<=39)){receive_no=receive_no | 0x04;}
	if((rec[5]>=26)&&(rec[5]<=39)){receive_no=receive_no | 0x08;}
	if((rec[6]>=26)&&(rec[6]<=39)){receive_no=receive_no | 0x10;}
	if((rec[7]>=26)&&(rec[7]<=39)){receive_no=receive_no | 0x20;}
	if((rec[8]>=26)&&(rec[8]<=39)){receive_no=receive_no | 0x40;}
	if((rec[9]>=26)&&(rec[9]<=39)){receive_no=receive_no | 0x80;}

}


/*---------- MAIN -----------*/
void	main( void )
{
unsigned int	sub;
unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw3_off;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;

unsigned int	nn;

unsigned char	auto_scan;
unsigned char	scan_flg;

unsigned int	vol_count;

	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;sw3_off=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;
	auto_scan=0;scan_flg=0;
	mode=1;preset_no=0;
	POWER_CONT=1;
	FM_RESET=1;
	count10_flg=0;

	wait_time(50000);
	
	vol_count=VOL_COUNT_SET;
	
	AD_Start();
	
	lcd_init();
	lcd_clear();

	lcd_xy(0,1);
	lcd_print("NEW FM RECEIVER ");
	fm_chip_init();
	mute(1);

	if((KEY_DOWN==0)&(KEY_MODE==0)){
		eeprom_init();
	}
	else{
		eeprom_check();
	}

	wait_time(50000);
	wait_time(50000);
	
	fm_chip_init();
	mute(1);

	lcd_xy(0,1);
	lcd_print("                ");
	lcd_ini_disp();
	
	while(KEY_POWER==0){}
	wait_time(10000);
	
	sub=0;
	tun_flg=0;
	kyoku_count=0;
	mute(0);
	
	preset_ch_init();
	preset_ch_set();

	switch(mode){
		case 0:	mhz_no=preset[preset_no];
				lcd_xy(4,0);
				lcd_mhz(mhz_no);
				lcd_kyokumei_clr();
				lcd_mode(mode);
				fm_data_set(mhz_no);
				kyoku_count=KYOKU_INIT;
				/*lcd_kyokumei(mhz_no);*/
				break;
		case 1:	if(tun_flg==1){
					mhz_no=vol_yomi();
				}
				mhz_no_m=mhz_no;
				lcd_xy(4,0);
				lcd_mhz(mhz_no);
				lcd_kyokumei_clr();
				lcd_ch_disp();
				lcd_mode(mode);
				fm_data_set(mhz_no);
				kyoku_count=KYOKU_INIT;
				/*lcd_kyokumei(mhz_no);*/
				break;
	}

	while(KEY_POWER==0){}

	TM50_Start();

	/* Start user code. Do not edit comment generated here */
	while (1) {
	/* key scan 1 */
		if (KEY_UP==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (KEY_DOWN==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (KEY_MODE==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
		}
		/*------------------*/
		if (KEY_POWER==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/


		wait_time(3000);
		/*wait_time(1000);*/
		



	/* key scan 2 */
		if ((KEY_UP==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((KEY_DOWN==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((KEY_MODE==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			/*sw3_cnt=0;*/
			if(sw3_cnt>0){
				sw3_off=1;
			}
		}
		/*------------------*/
		if ((KEY_POWER==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/


	/* key action */
	
		if((sw1_key==0)&&(sw1_rep==0)){
			switch(mode){
				case 0:
						sw1_rep=1;
						preset_no=preset_next(preset_no);
						mhz_no=preset[preset_no];
						lcd_xy(4,0);
						lcd_mhz(mhz_no);
						lcd_kyokumei_clr();
						lcd_mode(mode);
						fm_data_set(mhz_no);
						kyoku_count=KYOKU_INIT;
						/*lcd_kyokumei(mhz_no);*/
						sw1_cnt=0;
						write_direct();
						break;
				case 1:	
						sw1_rep=1;
						if(tun_flg==0){
							mhz_inc();
							lcd_xy(4,0);
							lcd_mhz(mhz_no);
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							mhz_no_m=mhz_no;
							fm_data_set(mhz_no);
							kyoku_count=KYOKU_INIT;
							/*lcd_kyokumei(mhz_no);*/
							sw1_cnt=0;
							write_direct();
						}
						break;
				case 2:
						sw1_rep=1;
						preset_no_setmode++;
						if(preset_no_setmode>=PRESET_MAX){preset_no_setmode=1;}
						mhz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(mhz_no_setmode!=0){
							lcd_mhz(mhz_no_setmode);
							lcd_print("MHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						sw1_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*------------------*/
		if((sw1_key==0)&&(sw1_rep==1)){
			sw1_cnt++;
			if(sw1_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							if(tun_flg==0){
								mhz_inc_10();
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_mode(mode);
								mhz_no_m=mhz_no;
								fm_data_set(mhz_no);
								kyoku_count=KYOKU_INIT;
								/*lcd_kyokumei(mhz_no);*/
								sw1_cnt=0;
								write_direct();
							}
							break;
					case 2:	break;
				}
			}
		}
		/*------------------*/
		if((sw2_key==0)&&(sw2_rep==0)){
			switch(mode){
				case 0:
						sw2_rep=1;
						preset_no=preset_back(preset_no);
						mhz_no=preset[preset_no];
						lcd_xy(4,0);
						lcd_mhz(mhz_no);
						lcd_kyokumei_clr();
						lcd_mode(mode);
						fm_data_set(mhz_no);
						kyoku_count=KYOKU_INIT;
						/*lcd_kyokumei(mhz_no);*/
						sw2_cnt=0;
						write_direct();
						break;
				case 1:
						sw2_rep=1;
						if(tun_flg==0){
							mhz_dec();
							lcd_xy(4,0);
							lcd_mhz(mhz_no);
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							mhz_no_m=mhz_no;
							fm_data_set(mhz_no);
							kyoku_count=KYOKU_INIT;
							/*lcd_kyokumei(mhz_no);*/
							sw2_cnt=0;
							write_direct();
						}
						break;
				case 2:
						sw2_rep=1;
						if(preset_no_setmode<=1){preset_no_setmode=PRESET_MAX-1;}
						else{preset_no_setmode--;}
						mhz_no_setmode=preset[preset_no_setmode];
						lcd_xy(8,1);
						if(mhz_no_setmode!=0){
							lcd_mhz(mhz_no_setmode);
							lcd_print("MHz");
						}
						else{
							lcd_print("=NONE=  ");
						}
						sw2_cnt=0;
						lcd_mode(mode);
						break;
			}
		}
		/*--------------------*/
		if((sw2_key==0)&&(sw2_rep==1)){
			sw2_cnt++;
			if(sw2_cnt>=30){
				switch(mode){
					case 0:	break;
					case 1:
							if(tun_flg==0){
								mhz_dec_10();
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_mode(mode);
								mhz_no_m=mhz_no;
								fm_data_set(mhz_no);
								kyoku_count=KYOKU_INIT;
								/*lcd_kyokumei(mhz_no);*/
								sw2_cnt=0;
								write_direct();
							}
							break;
					case 2:	break;
				}
			}
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			sw3_rep=1;
		}
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==1)){
			sw3_cnt++;
		}
		/*--------------------*/
		if((sw3_key==1)&&(sw3_off==1)){
			if(sw3_cnt>=40){
				/**/
				switch(mode){
					case 0:	break;
					case 1:
							mode=2;
							preset_no_setmode=preset_no;
							mhz_no_setmode=preset[preset_no];
							lcd_xy(8,1);
							lcd_mhz(mhz_no_setmode);
							lcd_print("MHz");
							sw1_cnt=0;
							lcd_mode(mode);
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=1;
								preset_no=preset_no_setmode;
								preset[preset_no]=0;
								preset_ch[mhz_no-FREQ_MIN]=0xff;
								if(tun_flg==1){
									mhz_no=vol_yomi();
								}

								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_kyokumei_clr();
								lcd_ch_disp();
								lcd_mode(mode);
								fm_data_set(mhz_no);
								/*lcd_kyokumei(mhz_no);*/

								write_direct();
								write_direct_set(preset_no);
								preset_no=preset_next(preset_no);
							}
							break;
				}
			}
			else{
				/*Z*/
				switch(mode){
					case 0:
							mode=1;
							if(tun_flg==1){
								mhz_no=vol_yomi();
							}

							lcd_xy(4,0);
							lcd_mhz(mhz_no);
							lcd_kyokumei_clr();
							lcd_ch_disp();
							lcd_mode(mode);
							fm_data_set(mhz_no);
							kyoku_count=KYOKU_INIT;
							/*lcd_kyokumei(mhz_no);*/

							write_direct();
							break;
					case 1:
							mode=0;
							mhz_no=preset[preset_no];
							lcd_xy(4,0);
							lcd_mhz(mhz_no);
							lcd_kyokumei_clr();
							lcd_mode(mode);
							fm_data_set(mhz_no);
							kyoku_count=KYOKU_INIT;
							/*lcd_kyokumei(mhz_no);*/
							write_direct();
							break;
					case 2:
							if(preset_no_setmode!=0){
								mode=0;
								preset_no=preset_no_setmode;
								preset[preset_no]=mhz_no;
								preset_ch[mhz_no-FREQ_MIN]=preset_no;
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_kyokumei_clr();
								lcd_mode(mode);
								fm_data_set(mhz_no);
								/*lcd_kyokumei(mhz_no);*/
								write_direct();
								write_direct_set(preset_no);
							}
							break;
				}
			}
			
			sw3_off=0;
			sw3_cnt=0;
		}
		/*--------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			switch(mode){
				case 0:	sw4_rep=1;
						break;
				case 1:	sw4_rep=1;
						switch(tun_flg){
							case 0:	tun_flg=1;
									mhz_no=vol_yomi();
									break;
							case 1:	tun_flg=0;
									break;
						}

						lcd_xy(4,0);
						lcd_mhz(mhz_no);
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						fm_data_set(mhz_no);
						kyoku_count=KYOKU_INIT;
						/*lcd_kyokumei(mhz_no);*/
						break;
				case 2:	sw4_rep=1;
						mode=1;
						lcd_xy(4,0);
						lcd_mhz(mhz_no);
						lcd_kyokumei_clr();
						lcd_ch_disp();
						lcd_mode(mode);
						fm_data_set(mhz_no);
						kyoku_count=KYOKU_INIT;
						/*lcd_kyokumei(mhz_no);*/
						write_direct();
						break;
			}
		}
		/*------------------*/
		if((sw4_key==0)&&(sw4_rep==1)){
			sw4_cnt++;
			if(sw4_cnt>=30){
				switch(mode){
					case 0:
					case 1:
							lcd_xy(0,1);
							lcd_print("                ");
							lcd_xy(0,0);
							lcd_print("=== POWER OFF===");
							mute(1);
							POWER_CONT=0;
							while(KEY_POWER==0){}
							break;
					case 2:	break;
				}
			}
		}
		/*--------------------*/



		/*------------------ */
		if((mode==1)&(tun_flg==1)){
		/*----------*/
		if(vol_count>0){vol_count--;}
		if(vol_count==0){
			mhz_no=vol_yomi();
			if(mhz_no>FREQ_MAX){mhz_no=FREQ_MAX;}
			if(mhz_no_m!=mhz_no){
				lcd_xy(4,0);
				lcd_mhz(mhz_no);
				lcd_kyokumei_clr();
				lcd_ch_disp();
				lcd_mode(mode);
				fm_data_set(mhz_no);
				kyoku_count=KYOKU_INIT;
				/*lcd_kyokumei(mhz_no);*/
				mhz_no_m=mhz_no;
			}
			vol_count=VOL_COUNT_SET;
		}
		}
		

		if(kyoku_count>0){
			kyoku_count--;
			if(kyoku_count==KYOKU_ON){
				switch(mode){
					case 0:
					case 1:	lcd_xy(8,1);
							lcd_kyokumei(mhz_no);
							break;
				}
			}
			if(kyoku_count==0){
				switch(mode){
					case 0:	
							break;
					case 1:	lcd_kyokumei_clr();
							lcd_ch_disp_sub();
							break;
				}
			}
		}


	/* receive data hantei */
	
		if(count10_flg==1){
			count10_flg=0;
			receive_hantei10();
			if(receive_err==0){
				switch(receive_no){
					case 0x20:	lcd_xy(0,1);
								lcd_print("                ");
								lcd_xy(0,0);
								lcd_print("=== POWER OFF===");
								mute(1);
								POWER_CONT=0;
								while(KEY_POWER==0){}
								break;
					case 0x40:	preset_no=preset_next(preset_no);
								mhz_no=preset[preset_no];
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_kyokumei_clr();
								lcd_mode(mode);
								fm_data_set(mhz_no);
								kyoku_count=KYOKU_INIT;
								/*lcd_kyokumei(mhz_no);*/
								write_direct();
								break;
					case 0x41:	preset_no=preset_back(preset_no);
								mhz_no=preset[preset_no];
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_kyokumei_clr();
								lcd_mode(mode);
								fm_data_set(mhz_no);
								kyoku_count=KYOKU_INIT;
								/*lcd_kyokumei(mhz_no);*/
								write_direct();
								break;
					case 0x50:	mode=0;
								mhz_no=preset[preset_no];
								lcd_xy(4,0);
								lcd_mhz(mhz_no);
								lcd_kyokumei_clr();
								lcd_mode(mode);
								fm_data_set(mhz_no);
								kyoku_count=KYOKU_INIT;
								/*lcd_kyokumei(mhz_no);*/
								write_direct();
								break;
				}
			}
		}


	}
	/* End user code. Do not edit comment generated here */
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */







